package com.visios.lipsync_spring_helper.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;

import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestTemplate;

import com.visios.lipsync_spring_helper.dto.ProcessRequestDto;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

@Service
public class ProcessingService {

    @Autowired
    private final RestTemplate restTemplate;

    public ProcessingService(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }
   
    @Async
    public CompletableFuture<Map<String, Object>> processPipeline(ProcessRequestDto request) {
        Map<String, Object> response = new HashMap<>();

        String text = request.getText();
        String speaker = request.getSpeaker();
        String email = request.getEmail();
        String model = request.getPerson_model();
        List<String> imageUrls = request.getImage_urls();
        String bgImage = request.getBg_image();
        String bgMusic = request.getBg_music();

        // Step 1: Generate Audio
        String audioUrl = "http://10.80.12.14:3335/generate_audio";
        Map<String, String> audioRequest = Map.of("text", text, "speaker", speaker);
        ResponseEntity<Map> audioResponse = restTemplate.postForEntity(audioUrl, audioRequest, Map.class);

        if (!audioResponse.getStatusCode().is2xxSuccessful()) {
            response.put("error", "An error occurred during generating audio");
            return CompletableFuture.completedFuture(response);
        }

        // Step 2: Generate Lipsync
        String lipsyncUrl = "http://10.80.12.14:3336/generate_lipsync";
        Map<String, String> lipsyncRequest = Map.of("person_model", model);
        ResponseEntity<Map> lipsyncResponse = restTemplate.postForEntity(lipsyncUrl, lipsyncRequest, Map.class);

        if (!lipsyncResponse.getStatusCode().is2xxSuccessful()) {
            response.put("error", "An error occurred during generating lipsync");
            return CompletableFuture.completedFuture(response);
        }

        // Step 3: Process Pipeline
        String pipelineUrl = "http://10.80.12.14:3334/process_pipeline";
        Map<String, Object> pipelineRequest = Map.of(
                "image_urls", imageUrls,
                "email", email,
                "bg_image", bgImage,
                "person_model", model,
                "bg_music", bgMusic
        );
        ResponseEntity<Map> pipelineResponse = restTemplate.postForEntity(pipelineUrl, pipelineRequest, Map.class);

        if (!pipelineResponse.getStatusCode().is2xxSuccessful()) {
            response.put("error", "An error occurred during processing pipeline");
            return CompletableFuture.completedFuture(response);
        }

        // Success
        response.put("message", "All steps completed successfully");
        return CompletableFuture.completedFuture(response);
    }
}

